@@ -29,6 +29,7 @@ from qbittorrentapi.exceptions import HTTP5XXError
 from qbittorrentapi.exceptions import HTTPError
 from qbittorrentapi.exceptions import InternalServerError500Error
 from qbittorrentapi.exceptions import InvalidRequest400Error
+from qbittorrentapi.exceptions import MethodNotAllowed405Error
 from qbittorrentapi.exceptions import MissingRequiredParameters400Error
 from qbittorrentapi.exceptions import NotFound404Error
 from qbittorrentapi.exceptions import Unauthorized401Error
@@ -48,8 +49,8 @@ class URL(object):
         """
         Create a fully qualified URL for the API endpoint.
 
-        :param api_namespace: the namespace for the API endpoint (e.g. torrents)
-        :param api_method: the specific method for the API endpoint (e.g. info)
+        :param api_namespace: the namespace for the API endpoint (e.g. ``torrents``)
+        :param api_method: the specific method for the API endpoint (e.g. ``info``)
         :param headers: HTTP headers for request
         :param requests_kwargs: kwargs for any calls to Requests
         :return: fully qualified URL string for endpoint
@@ -79,8 +80,8 @@ class URL(object):
         and prefixed to all subsequent API calls.
 
         :param headers: HTTP headers for request
-        :param requests_kwargs: additional parameters from user for making HTTP HEAD request
-        :return: base URL as string for Web API endpoint
+        :param requests_kwargs: additional params from user for HTTP ``HEAD`` request
+        :return: base URL as a ``string`` for Web API endpoint
         """
         if self.client._API_BASE_URL is not None:
             return self.client._API_BASE_URL
@@ -133,12 +134,12 @@ class URL(object):
         """
         Determine if the URL endpoint is using HTTP or HTTPS.
 
-        :param base_url: urllib URL object
+        :param base_url: urllib :class:`~urllib.parse.ParseResult` URL object
         :param default_scheme: default scheme to use for URL
         :param alt_scheme: alternative scheme to use for URL if default doesn't work
         :param headers: HTTP headers for request
         :param requests_kwargs: kwargs for calls to Requests
-        :return: scheme (ie HTTP or HTTPS)
+        :return: scheme (ie ``HTTP`` or ``HTTPS``)
         """
         logger.debug("Detecting scheme for URL...")
         prefer_https = False
@@ -171,7 +172,8 @@ class URL(object):
         :param api_namespace: the namespace for the API endpoint (e.g. torrents)
         :param api_method: the specific method for the API endpoint (e.g. info)
         :return: entire URL string for API endpoint
-                 (e.g. http://localhost:8080/api/v2/torrents/info or http://example.com/qbt/api/v2/torrents/info)
+                 (e.g. ``http://localhost:8080/api/v2/torrents/info``
+                 or ``http://example.com/qbt/api/v2/torrents/info``)
         """
         if isinstance(api_namespace, APINames):
             api_namespace = api_namespace.value
@@ -271,34 +273,37 @@ class Request(object):
                 if getLogger(logger_).level < 20:
                     getLogger(logger_).setLevel("INFO")
 
-        # Environment variables have lowest priority
-        if self.host == "" and environ.get("PYTHON_QBITTORRENTAPI_HOST") is not None:
-            logger.debug(
-                "Using PYTHON_QBITTORRENTAPI_HOST env variable for qBittorrent hostname"
+        # Environment variables have the lowest priority
+        if not self.host:
+            env_host = environ.get(
+                "QBITTORRENTAPI_HOST", environ.get("PYTHON_QBITTORRENTAPI_HOST")
             )
-            self.host = environ["PYTHON_QBITTORRENTAPI_HOST"]
-        if (
-            self.username == ""
-            and environ.get("PYTHON_QBITTORRENTAPI_USERNAME") is not None
-        ):
-            logger.debug(
-                "Using PYTHON_QBITTORRENTAPI_USERNAME env variable for username"
+            if env_host is not None:
+                logger.debug(
+                    "Using QBITTORRENTAPI_HOST env variable for qBittorrent host"
+                )
+                self.host = env_host
+        if not self.username:
+            env_username = environ.get(
+                "QBITTORRENTAPI_USERNAME", environ.get("PYTHON_QBITTORRENTAPI_USERNAME")
             )
-            self.username = environ["PYTHON_QBITTORRENTAPI_USERNAME"]
-        if (
-            self._password == ""
-            and environ.get("PYTHON_QBITTORRENTAPI_PASSWORD") is not None
-        ):
-            logger.debug(
-                "Using PYTHON_QBITTORRENTAPI_PASSWORD env variable for password"
+            if env_username is not None:
+                logger.debug("Using QBITTORRENTAPI_USERNAME env variable for username")
+                self.username = env_username
+        if not self._password:
+            env_password = environ.get(
+                "QBITTORRENTAPI_PASSWORD", environ.get("PYTHON_QBITTORRENTAPI_PASSWORD")
             )
-            self._password = environ["PYTHON_QBITTORRENTAPI_PASSWORD"]
-        if (
-            self._VERIFY_WEBUI_CERTIFICATE is True
-            and environ.get("PYTHON_QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE")
-            is not None
-        ):
-            self._VERIFY_WEBUI_CERTIFICATE = False
+            if env_password is not None:
+                logger.debug("Using QBITTORRENTAPI_PASSWORD env variable for password")
+                self._password = env_password
+        if self._VERIFY_WEBUI_CERTIFICATE is True:
+            env_verify_cert = environ.get(
+                "QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE",
+                environ.get("PYTHON_QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"),
+            )
+            if env_verify_cert is not None:
+                self._VERIFY_WEBUI_CERTIFICATE = False
 
         # Mocking variables until better unit testing exists
         self._MOCK_WEB_API_VERSION = MOCK_WEB_API_VERSION
@@ -319,11 +324,29 @@ class Request(object):
         return input_list
 
     def _get(self, _name=APINames.EMPTY, _method="", **kwargs):
+        """
+        Send ``GET`` request.
+
+        :param api_namespace: the namespace for the API endpoint
+            (e.g. :class:`~qbittorrentapi.definitions.APINames` or ``torrents``)
+        :param api_method: the name for the API endpoint (e.g. ``add``)
+        :param kwargs: see :meth:`~Request._request`
+        :return: Requests :class:`~requests.Response`
+        """
         return self._request_manager(
             http_method="get", api_namespace=_name, api_method=_method, **kwargs
         )
 
     def _post(self, _name=APINames.EMPTY, _method="", **kwargs):
+        """
+        Send ``POST`` request.
+
+        :param api_namespace: the namespace for the API endpoint
+            (e.g. :class:`~qbittorrentapi.definitions.APINames` or ``torrents``)
+        :param api_method: the name for the API endpoint (e.g. ``add``)
+        :param kwargs: see :meth:`~Request._request`
+        :return: Requests :class:`~requests.Response`
+        """
         return self._request_manager(
             http_method="post", api_namespace=_name, api_method=_method, **kwargs
         )
@@ -373,7 +396,7 @@ class Request(object):
             logger.debug("Retry attempt %d", retry_count + 1)
 
         max_retries = _retries if _retries >= 1 else 1
-        for retry in range(0, (max_retries + 1)):
+        for retry in range(0, (max_retries + 1)):  # pragma: no branch
             try:
                 return self._request(**kwargs)
             except HTTPError as exc:
@@ -406,17 +429,18 @@ class Request(object):
         """
         Meat and potatoes of sending requests to qBittorrent.
 
-        :param http_method: 'get' or 'post'
-        :param api_namespace: the namespace for the API endpoint (e.g. torrents)
-        :param api_method: the namespace for the API endpoint (e.g. torrents)
+        :param http_method: ``get`` or ``post``
+        :param api_namespace: the namespace for the API endpoint
+            (e.g. :class:`~qbittorrentapi.definitions.APINames` or ``torrents``)
+        :param api_method: the name for the API endpoint (e.g. ``add``)
         :param requests_args: default location for Requests kwargs
         :param requests_params: alternative location for Requests kwargs
         :param headers: HTTP headers to send with the request
-        :param params: key/value pairs to send with a GET request
-        :param data: key/value pairs to send with a POST request
+        :param params: key/value pairs to send with a ``GET`` request
+        :param data: key/value pairs to send with a ``POST`` request
         :param files: files to be sent with the request
         :param kwargs: arbitrary keyword args to send to qBittorrent with the request
-        :return: Requests response
+        :return: Requests :class:`~requests.Response`
         """
         kwargs = self._trim_known_kwargs(**kwargs)
 
@@ -458,12 +482,12 @@ class Request(object):
     def _get_requests_kwargs(self, requests_args=None, requests_params=None):
         """
         Determine the requests_kwargs for the call to Requests. The global
-        configuration in self._REQUESTS_ARGS is updated by any arguments
+        configuration in ``self._REQUESTS_ARGS`` is updated by any arguments
         provided for a specific call.
 
-        :param requests_args: default location to expect Requests requests_kwargs
-        :param requests_params: alternative location to expect Requests requests_kwargs
-        :return: final dictionary of Requests requests_kwargs
+        :param requests_args: default location to expect Requests ``requests_kwargs``
+        :param requests_params: alternative location to expect Requests ``requests_kwargs``
+        :return: final dictionary of Requests ``requests_kwargs``
         """
         requests_kwargs = deepcopy(self._REQUESTS_ARGS)
         requests_kwargs.update(requests_args or requests_params or {})
@@ -473,7 +497,7 @@ class Request(object):
         """
         Determine headers specific to this request. Request headers can be
         specified explicitly or with the requests kwargs. Headers specified in
-        self._EXTRA_HEADERS are merged in Requests itself.
+        ``self._EXTRA_HEADERS`` are merged in Requests itself.
 
         :param headers: headers specified for this specific request
         :param more_headers: headers from requests_kwargs config
@@ -487,7 +511,7 @@ class Request(object):
         """
         Determine data, params, and files for the Requests call.
 
-        :param http_method: get or post
+        :param http_method: ``get`` or ``post``
         :param params: key value pairs to send with GET calls
         :param data: key value pairs to send with POST calls
         :param files: dictionary of files to send with request
@@ -513,7 +537,7 @@ class Request(object):
         """
         Create or return existing HTTP session.
 
-        :return: Requests Session object
+        :return: Requests :class:`~requests.Session` object
         """
 
         class QbittorrentSession(Session):
@@ -600,18 +624,18 @@ class Request(object):
         if response.status_code < 400:
             # short circuit for non-error statuses
             return
+
         if response.status_code == 400:
             # Returned for malformed requests such as missing or invalid parameters.
-            #
             # If an error_message isn't returned, qBittorrent didn't receive all required parameters.
             # APIErrorType::BadParams
-            # the name (i.e. Bad Response) of the HTTP error started being returned in v4.3.0
+            # the name of the HTTP error (i.e. Bad Request) started being returned in v4.3.0
             if response.text in ("", "Bad Request"):
                 raise MissingRequiredParameters400Error()
             raise InvalidRequest400Error(response.text)
 
         if response.status_code == 401:
-            # Primarily reserved for XSS and host header issues. Is also
+            # Primarily reserved for XSS and host header issues.
             raise Unauthorized401Error(response.text)
 
         if response.status_code == 403:
@@ -624,17 +648,17 @@ class Request(object):
             # APIErrorType::NotFound
             error_message = response.text
             if error_message in ("", "Not Found"):
-                error_torrent_hash = ""
-                if data:
-                    error_torrent_hash = data.get("hash", error_torrent_hash)
-                    error_torrent_hash = data.get("hashes", error_torrent_hash)
-                if params and error_torrent_hash == "":
-                    error_torrent_hash = params.get("hash", error_torrent_hash)
-                    error_torrent_hash = params.get("hashes", error_torrent_hash)
-                if error_torrent_hash:
-                    error_message = "Torrent hash(es): %s" % error_torrent_hash
+                hash_source = data or params or {}
+                error_hash = hash_source.get("hashes", hash_source.get("hash", ""))
+                if error_hash:
+                    error_message = "Torrent hash(es): %s" % error_hash
             raise NotFound404Error(error_message)
 
+        if response.status_code == 405:
+            # HTTP method not allowed for the API endpoint.
+            # This should only be raised if qBittorrent changes the requirement for an endpoint...
+            raise MethodNotAllowed405Error(response.text)
+
         if response.status_code == 409:
             # APIErrorType::Conflict
             raise Conflict409Error(response.text)
@@ -644,11 +668,14 @@ class Request(object):
             raise UnsupportedMediaType415Error(response.text)
 
         if response.status_code >= 500:
-            raise InternalServerError500Error(response.text)
+            http_error = InternalServerError500Error(response.text)
+            http_error.http_status_code = response.status_code
+            raise http_error
 
-        if response.status_code >= 400:
-            # Unaccounted for errors from API
-            raise HTTPError(response.text)
+        # Unaccounted for API errors
+        http_error = HTTPError(response.text)
+        http_error.http_status_code = response.status_code
+        raise http_error
 
     def _verbose_logging(
         self, http_method, url, data, params, requests_kwargs, response
