@@ -1,4 +1,5 @@
-from qbittorrentapi.decorators import Alias
+from qbittorrentapi.app import AppAPIMixIn
+from qbittorrentapi.decorators import alias
 from qbittorrentapi.decorators import aliased
 from qbittorrentapi.decorators import endpoint_introduced
 from qbittorrentapi.decorators import login_required
@@ -9,7 +10,6 @@ from qbittorrentapi.definitions import ClientCache
 from qbittorrentapi.definitions import Dictionary
 from qbittorrentapi.definitions import List
 from qbittorrentapi.definitions import ListEntry
-from qbittorrentapi.request import Request
 
 
 class SearchJobDictionary(Dictionary):
@@ -84,7 +84,7 @@ class SearchPlugin(ListEntry):
 @aliased
 class Search(ClientCache):
     """
-    Allows interaction with "Search" API endpoints.
+    Allows interaction with ``Search`` API endpoints.
 
     :Usage:
         >>> from qbittorrentapi import Client
@@ -136,33 +136,33 @@ class Search(ClientCache):
         """Implements :meth:`~SearchAPIMixIn.search_plugins`"""
         return self._client.search_plugins()
 
-    @Alias("installPlugin")
+    @alias("installPlugin")
     def install_plugin(self, sources=None, **kwargs):
         """Implements :meth:`~SearchAPIMixIn.search_install_plugin`"""
         return self._client.search_install_plugin(sources=sources, **kwargs)
 
-    @Alias("uninstallPlugin")
+    @alias("uninstallPlugin")
     def uninstall_plugin(self, sources=None, **kwargs):
         """Implements :meth:`~SearchAPIMixIn.search_uninstall_plugin`"""
         return self._client.search_uninstall_plugin(sources=sources, **kwargs)
 
-    @Alias("enablePlugin")
+    @alias("enablePlugin")
     def enable_plugin(self, plugins=None, enable=None, **kwargs):
         """Implements :meth:`~SearchAPIMixIn.search_enable_plugin`"""
         return self._client.search_enable_plugin(
             plugins=plugins, enable=enable, **kwargs
         )
 
-    @Alias("updatePlugins")
+    @alias("updatePlugins")
     def update_plugins(self, **kwargs):
         """Implements :meth:`~SearchAPIMixIn.search_update_plugins`"""
         return self._client.search_update_plugins(**kwargs)
 
 
 @aliased
-class SearchAPIMixIn(Request):
+class SearchAPIMixIn(AppAPIMixIn):
     """
-    Implementation for all Search API methods.
+    Implementation for all ``Search`` API methods.
 
     :Usage:
         >>> from qbittorrentapi import Client
@@ -177,7 +177,7 @@ class SearchAPIMixIn(Request):
     @property
     def search(self):
         """
-        Allows for transparent interaction with Search endpoints.
+        Allows for transparent interaction with ``Search`` endpoints.
 
         See Search class for usage.
         :return: Search object
@@ -232,7 +232,7 @@ class SearchAPIMixIn(Request):
         :raises NotFound404Error:
 
         :param search_id: ID of search to get status; leave emtpy for status of all jobs
-        :return: :class:`SearchStatusesList` - https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-search-status
+        :return: :class:`SearchStatusesList` - `<https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-search-status>`_
         """  # noqa: E501
         params = {"id": search_id}
         return self._get(
@@ -252,7 +252,7 @@ class SearchAPIMixIn(Request):
         :param search_id: ID of search job
         :param limit: number of results to return
         :param offset: where to start returning results
-        :return: :class:`SearchResultsDictionary` - https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-search-results
+        :return: :class:`SearchResultsDictionary` - `<https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-search-results>`_
         """  # noqa: E501
         data = {"id": search_id, "limit": limit, "offset": offset}
         return self._post(_name=APINames.Search, _method="results", data=data, **kwargs)
@@ -278,9 +278,10 @@ class SearchAPIMixIn(Request):
     def search_categories(self, plugin_name=None, **kwargs):
         """
         Retrieve categories for search.
+
         Note: endpoint was removed in qBittorrent v4.3.0
 
-        :param plugin_name: Limit categories returned by plugin(s) (supports 'all' and 'enabled')
+        :param plugin_name: Limit categories returned by plugin(s) (supports ``all`` and ``enabled``)
         :return: :class:`SearchCategoriesList`
         """
         data = {"pluginName": plugin_name}
@@ -295,17 +296,16 @@ class SearchAPIMixIn(Request):
         """
         Retrieve details of search plugins.
 
-        :return: :class:`SearchPluginsList` - https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-search-plugins
+        :return: :class:`SearchPluginsList` - `<https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-search-plugins>`_
         """  # noqa: E501
         return self._get(_name=APINames.Search, _method="plugins", **kwargs)
 
     @endpoint_introduced("2.1.1", "search/installPlugin")
-    @Alias("search_installPlugin")
+    @alias("search_installPlugin")
     @login_required
     def search_install_plugin(self, sources=None, **kwargs):
         """
-        Install search plugins from either URL or file. (alias:
-        search_installPlugin)
+        Install search plugins from either URL or file.
 
         :param sources: list of URLs or filepaths
         :return: None
@@ -314,11 +314,11 @@ class SearchAPIMixIn(Request):
         self._post(_name=APINames.Search, _method="installPlugin", data=data, **kwargs)
 
     @endpoint_introduced("2.1.1", "search/uninstallPlugin")
-    @Alias("search_uninstallPlugin")
+    @alias("search_uninstallPlugin")
     @login_required
     def search_uninstall_plugin(self, names=None, **kwargs):
         """
-        Uninstall search plugins. (alias: search_uninstallPlugin)
+        Uninstall search plugins.
 
         :param names: names of plugins to uninstall
         :return: None
@@ -329,11 +329,11 @@ class SearchAPIMixIn(Request):
         )
 
     @endpoint_introduced("2.1.1", "search/enablePlugin")
-    @Alias("search_enablePlugin")
+    @alias("search_enablePlugin")
     @login_required
     def search_enable_plugin(self, plugins=None, enable=None, **kwargs):
         """
-        Enable or disable search plugin(s). (alias: search_enablePlugin)
+        Enable or disable search plugin(s).
 
         :param plugins: list of plugin names
         :param enable: True or False
@@ -343,11 +343,11 @@ class SearchAPIMixIn(Request):
         self._post(_name=APINames.Search, _method="enablePlugin", data=data, **kwargs)
 
     @endpoint_introduced("2.1.1", "search/updatePlugin")
-    @Alias("search_updatePlugins")
+    @alias("search_updatePlugins")
     @login_required
     def search_update_plugins(self, **kwargs):
         """
-        Auto update search plugins. (alias: search_updatePlugins)
+        Auto update search plugins.
 
         :return: None
         """
