@@ -1,8 +1,7 @@
-import os
-
 import pytest
 from pkg_resources import parse_version as v
 
+from qbittorrentapi._attrdict import AttrDict
 from tests.conftest import IS_QBT_DEV
 
 
@@ -37,12 +36,12 @@ def test_preferences(client):
     assert "dht" in prefs
     assert "dht" in client.app.preferences
     dht = prefs["dht"]
-    client.app.preferences = dict(dht=(not dht))
+    client.app.preferences = AttrDict(dht=(not dht))
     assert dht is not client.app.preferences.dht
     client.app_set_preferences(prefs=dict(dht=dht))
     assert dht is client.app.preferences.dht
 
 
 def test_default_save_path(client):
-    assert os.path.isdir(client.app_default_save_path())
-    assert os.path.isdir(client.app.default_save_path)
+    assert "download" in client.app_default_save_path().lower()
+    assert "download" in client.app.default_save_path.lower()
