@@ -1,22 +1,30 @@
 import logging
-import os
 import re
 import sys
-from collections import namedtuple
 from os import environ
 
+try:
+    from unittest.mock import MagicMock
+except ImportError:  # python 2
+    from mock import MagicMock
+
 import pytest
 from pkg_resources import parse_version as v
+from requests import Response
 
+from qbittorrentapi import APINames
 from qbittorrentapi import Client
 from qbittorrentapi import exceptions
 from qbittorrentapi.request import Request
 from qbittorrentapi.torrents import TorrentDictionary
 from qbittorrentapi.torrents import TorrentInfoList
-from tests.conftest import BASE_PATH
 from tests.conftest import IS_QBT_DEV
+from tests.conftest import mkpath
 
-MockResponse = namedtuple("MockResponse", ("status_code", "text"))
+
+def test_method_name(client, app_version):
+    assert app_version == client._get("app", "version").text
+    assert app_version == client._get(APINames.Application, "version").text
 
 
 def test_log_in():
@@ -46,8 +54,8 @@ def test_log_in_via_auth():
 
     assert (
         client_good.auth_log_in(
-            username=environ.get("PYTHON_QBITTORRENTAPI_USERNAME"),
-            password=environ.get("PYTHON_QBITTORRENTAPI_PASSWORD"),
+            username=environ.get("QBITTORRENTAPI_USERNAME"),
+            password=environ.get("QBITTORRENTAPI_PASSWORD"),
         )
         is None
     )
@@ -94,7 +102,7 @@ def test_hostname_user_base_path(hostname):
 
 
 def test_port_from_host(app_version):
-    host, port = environ.get("PYTHON_QBITTORRENTAPI_HOST").split(":")
+    host, port = environ.get("QBITTORRENTAPI_HOST").split(":")
     client = Client(host=host, port=port, VERIFY_WEBUI_CERTIFICATE=False)
     assert client.app.version == app_version
 
@@ -103,89 +111,137 @@ def _enable_disable_https(client, use_https):
     if use_https:
         client.app.preferences = {
             "use_https": True,
-            "web_ui_https_cert_path": os.path.join(
-                BASE_PATH, "tests", "resources", "server.crt"
-            ),
-            "web_ui_https_key_path": os.path.join(
-                BASE_PATH, "tests", "resources", "server.key"
-            ),
+            "web_ui_https_cert_path": mkpath("/tmp", "resources", "server.crt"),
+            "web_ui_https_key_path": mkpath("/tmp", "resources", "server.key"),
         }
     else:
         client.app.preferences = {"use_https": False}
 
 
 @pytest.mark.parametrize("use_https", (True, False))
-def test_force_user_scheme(client, app_version, use_https):
-    default_host = environ["PYTHON_QBITTORRENTAPI_HOST"]
+def test_force_user_scheme(client, app_version, api_version, use_https):
+    if v(api_version) >= v("2.2.1"):
+        default_host = environ["QBITTORRENTAPI_HOST"]
 
-    _enable_disable_https(client, use_https)
+        _enable_disable_https(client, use_https)
 
-    client = Client(
-        host="http://" + default_host,
-        VERIFY_WEBUI_CERTIFICATE=False,
-        FORCE_SCHEME_FROM_HOST=True,
-        REQUESTS_ARGS={"timeout": 3},
-    )
-    if use_https:
-        with pytest.raises(exceptions.APIConnectionError):
+        client = Client(
+            host="http://" + default_host,
+            VERIFY_WEBUI_CERTIFICATE=False,
+            FORCE_SCHEME_FROM_HOST=True,
+            REQUESTS_ARGS={"timeout": 3},
+        )
+        if use_https:
+            with pytest.raises(exceptions.APIConnectionError):
+                assert client.app.version == app_version
+        else:
             assert client.app.version == app_version
-    else:
-        assert client.app.version == app_version
-    assert client._API_BASE_URL.startswith("http://")
-
-    client = Client(
-        host=default_host,
-        VERIFY_WEBUI_CERTIFICATE=False,
-        FORCE_SCHEME_FROM_HOST=True,
-        REQUESTS_ARGS={"timeout": 3},
-    )
-    assert client.app.version == app_version
-    if use_https:
-        assert client._API_BASE_URL.startswith("https://")
-    else:
         assert client._API_BASE_URL.startswith("http://")
 
-    client = Client(
-        host="https://" + default_host,
-        VERIFY_WEBUI_CERTIFICATE=False,
-        FORCE_SCHEME_FROM_HOST=True,
-        REQUESTS_ARGS={"timeout": 3},
-    )
-    if not use_https:
-        with pytest.raises(exceptions.APIConnectionError):
-            assert client.app.version == app_version
-    else:
+        client = Client(
+            host=default_host,
+            VERIFY_WEBUI_CERTIFICATE=False,
+            FORCE_SCHEME_FROM_HOST=True,
+            REQUESTS_ARGS={"timeout": 3},
+        )
         assert client.app.version == app_version
-    assert client._API_BASE_URL.startswith("https://")
+        if use_https:
+            assert client._API_BASE_URL.startswith("https://")
+        else:
+            assert client._API_BASE_URL.startswith("http://")
+
+        client = Client(
+            host="https://" + default_host,
+            VERIFY_WEBUI_CERTIFICATE=False,
+            FORCE_SCHEME_FROM_HOST=True,
+            REQUESTS_ARGS={"timeout": 3},
+        )
+        if not use_https:
+            with pytest.raises(exceptions.APIConnectionError):
+                assert client.app.version == app_version
+        else:
+            assert client.app.version == app_version
+        assert client._API_BASE_URL.startswith("https://")
 
 
 @pytest.mark.parametrize("scheme", ("http://", "https://"))
-def test_both_https_http_not_working(client, app_version, scheme):
-    default_host = environ["PYTHON_QBITTORRENTAPI_HOST"]
-    _enable_disable_https(client, use_https=True)
+def test_both_https_http_not_working(client, app_version, api_version, scheme):
+    if v(api_version) >= v("2.2.1"):
+        default_host = environ["QBITTORRENTAPI_HOST"]
+        _enable_disable_https(client, use_https=True)
 
-    # rerun with verify=True
-    test_client = Client(
-        host=scheme + default_host,
-        REQUESTS_ARGS={"timeout": 3},
-    )
-    with pytest.raises(exceptions.APIConnectionError):
-        assert test_client.app.version == app_version
-    assert test_client._API_BASE_URL.startswith("https://")
+        # rerun with verify=True
+        test_client = Client(
+            host=scheme + default_host,
+            REQUESTS_ARGS={"timeout": 3},
+        )
+        with pytest.raises(exceptions.APIConnectionError):
+            assert test_client.app.version == app_version
+        assert test_client._API_BASE_URL.startswith("https://")
 
-    _enable_disable_https(client, use_https=False)
+        _enable_disable_https(client, use_https=False)
+
+
+def test_legacy_env_vars():
+    save_env_host = environ.get("QBITTORRENTAPI_HOST")
+    save_env_username = environ.get("QBITTORRENTAPI_USERNAME")
+    save_env_password = environ.get("QBITTORRENTAPI_PASSWORD")
+    save_env_verify = environ.get("QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE")
+    del environ["QBITTORRENTAPI_HOST"]
+    del environ["QBITTORRENTAPI_USERNAME"]
+    del environ["QBITTORRENTAPI_PASSWORD"]
+    if "QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE" in environ:
+        del environ["QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"]
+
+    client = Client()
+
+    assert client.host == ""
+    assert client.username == ""
+    assert client._password == ""
+    assert client._VERIFY_WEBUI_CERTIFICATE is True
+
+    environ["PYTHON_QBITTORRENTAPI_HOST"] = excepted_host = "legacy:8090"
+    environ["PYTHON_QBITTORRENTAPI_USERNAME"] = excepted_username = "legacyuser"
+    environ["PYTHON_QBITTORRENTAPI_PASSWORD"] = expected_password = "legacypassword"
+    environ["PYTHON_QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"] = "true"
+
+    client = Client()
+
+    try:
+        assert client.host == excepted_host
+        assert client.username == excepted_username
+        assert client._password == expected_password
+        assert client._VERIFY_WEBUI_CERTIFICATE is False
+    finally:
+        environ["QBITTORRENTAPI_HOST"] = save_env_host
+        environ["QBITTORRENTAPI_USERNAME"] = save_env_username
+        environ["QBITTORRENTAPI_PASSWORD"] = save_env_password
+        if save_env_verify is not None:
+            environ["QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"] = save_env_verify
+
+    client = Client()
+
+    assert client.host != excepted_host
+    assert client.username != excepted_username
+    assert client._password != expected_password
+    assert client._VERIFY_WEBUI_CERTIFICATE is False
+
+    del environ["PYTHON_QBITTORRENTAPI_HOST"]
+    del environ["PYTHON_QBITTORRENTAPI_USERNAME"]
+    del environ["PYTHON_QBITTORRENTAPI_PASSWORD"]
+    del environ["PYTHON_QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"]
 
 
 def test_log_out(client):
     client.auth_log_out()
     with pytest.raises(exceptions.Forbidden403Error):
         # cannot call client.app.version directly since it will auto log back in
-        client._get("app", "version")
+        client._get(APINames.Application, "version")
     client.auth_log_in()
     client.auth.log_out()
     with pytest.raises(exceptions.Forbidden403Error):
         # cannot call client.app.version directly since it will auto log back in
-        client._get("app", "version")
+        client._get(APINames.Application, "version")
     client.auth_log_in()
 
 
@@ -193,6 +249,9 @@ def test_port(app_version):
     client = Client(host="localhost", port=8080, VERIFY_WEBUI_CERTIFICATE=False)
     assert client.app.version == app_version
 
+    client = Client(host="localhost:8080", port=8081, VERIFY_WEBUI_CERTIFICATE=False)
+    assert client.app.version == app_version
+
 
 def test_simple_response(client, orig_torrent):
     torrent = client.torrents_info()[0]
@@ -219,16 +278,18 @@ def test_request_extra_headers():
     )
     client.auth.log_in()
 
-    r = client._get(_name="app", _method="version")
+    r = client._get(APINames.Application, "version")
     assert r.request.headers["X-MY-HEADER"] == "asdf"
 
-    r = client._get(_name="app", _method="version", headers={"X-MY-HEADER-TWO": "zxcv"})
+    r = client._get(
+        APINames.Application, "version", headers={"X-MY-HEADER-TWO": "zxcv"}
+    )
     assert r.request.headers["X-MY-HEADER"] == "asdf"
     assert r.request.headers["X-MY-HEADER-TWO"] == "zxcv"
 
     r = client._get(
-        _name="app",
-        _method="version",
+        APINames.Application,
+        "version",
         headers={"X-MY-HEADER-TWO": "zxcv"},
         requests_args={"headers": {"X-MY-HEADER-THREE": "tyui"}},
     )
@@ -237,20 +298,20 @@ def test_request_extra_headers():
     assert r.request.headers["X-MY-HEADER-THREE"] == "tyui"
 
     r = client._get(
-        _name="app",
-        _method="version",
+        APINames.Application,
+        "version",
         headers={"X-MY-HEADER": "zxcv"},
         requests_args={"headers": {"X-MY-HEADER": "tyui"}},
     )
     assert r.request.headers["X-MY-HEADER"] == "zxcv"
 
-    r = client._get(_name="app", _method="version", headers={"X-MY-HEADER": "zxcv"})
+    r = client._get(APINames.Application, "version", headers={"X-MY-HEADER": "zxcv"})
     assert r.request.headers["X-MY-HEADER"] == "zxcv"
 
 
-def test_requests_timeout():
+def test_requests_timeout(api_version):
     # timeouts are weird on python 2...just skip it...
-    if sys.version_info[0] < 3:
+    if sys.version_info[0] < 3 or v(api_version) < v("2.2.1"):
         return
 
     class MyTimeoutError(Exception):
@@ -285,12 +346,12 @@ def test_request_extra_params(client, orig_torrent_hash):
     """extra params can be sent directly to qBittorrent but there aren't any
     real use-cases so force it."""
     json_response = client._post(
-        _name="torrents", _method="info", hashes=orig_torrent_hash
+        APINames.Torrents, "info", hashes=orig_torrent_hash
     ).json()
     torrent = TorrentInfoList(json_response, client)[0]
     assert isinstance(torrent, TorrentDictionary)
     json_response = client._get(
-        _name="torrents", _method="info", hashes=orig_torrent_hash
+        APINames.Torrents, "info", hashes=orig_torrent_hash
     ).json()
     torrent = TorrentInfoList(json_response, client)[0]
     assert isinstance(torrent, TorrentDictionary)
@@ -311,28 +372,28 @@ def test_unsupported_version_error():
         RAISE_ERROR_FOR_UNSUPPORTED_QBITTORRENT_VERSIONS=True,
     )
     with pytest.raises(exceptions.UnsupportedQbittorrentVersion):
-        _ = client.app.version
+        client.app_version()
 
     client = Client(
         VERIFY_WEBUI_CERTIFICATE=False,
         RAISE_ERROR_FOR_UNSUPPORTED_QBITTORRENT_VERSIONS=True,
     )
-    _ = client.app.version
+    client.app_version()
 
 
 def test_disable_logging():
-    _ = Client(DISABLE_LOGGING_DEBUG_OUTPUT=False)
+    Client(DISABLE_LOGGING_DEBUG_OUTPUT=False)
     assert logging.getLogger("qbittorrentapi").level == logging.NOTSET
     assert logging.getLogger("requests").level == logging.NOTSET
     assert logging.getLogger("urllib3").level == logging.NOTSET
 
-    _ = Client(DISABLE_LOGGING_DEBUG_OUTPUT=True)
+    Client(DISABLE_LOGGING_DEBUG_OUTPUT=True)
     assert logging.getLogger("qbittorrentapi").level == logging.INFO
     assert logging.getLogger("requests").level == logging.INFO
     assert logging.getLogger("urllib3").level == logging.INFO
 
     logging.getLogger("qbittorrentapi").setLevel(level=logging.CRITICAL)
-    _ = Client(DISABLE_LOGGING_DEBUG_OUTPUT=True)
+    Client(DISABLE_LOGGING_DEBUG_OUTPUT=True)
     assert logging.getLogger("qbittorrentapi").level == logging.CRITICAL
     assert logging.getLogger("requests").level == logging.INFO
     assert logging.getLogger("urllib3").level == logging.INFO
@@ -348,10 +409,11 @@ def test_verify_cert(app_version):
     # assert client._VERIFY_WEBUI_CERTIFICATE is True  # noqa: E800
     # assert client.app.version == app_version  # noqa: E800
 
-    environ["PYTHON_QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"] = "true"
+    environ["QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"] = "true"
     client = Client(VERIFY_WEBUI_CERTIFICATE=True)
     assert client._VERIFY_WEBUI_CERTIFICATE is False
     assert client.app.version == app_version
+    del environ["QBITTORRENTAPI_DO_NOT_VERIFY_WEBUI_CERTIFICATE"]
 
 
 def test_api_connection_error():
@@ -372,11 +434,12 @@ def test_http400(client, app_version, orig_torrent_hash):
     with pytest.raises(exceptions.MissingRequiredParameters400Error):
         client.torrents_file_priority(hash=orig_torrent_hash)
 
-    if v(app_version) > v("4.1.5"):
-        with pytest.raises(exceptions.InvalidRequest400Error):
+    if v(app_version) >= v("4.1.6"):
+        with pytest.raises(exceptions.InvalidRequest400Error) as exc_info:
             client.torrents_file_priority(
                 hash=orig_torrent_hash, file_ids="asdf", priority="asdf"
             )
+        assert exc_info.value.http_status_code == 400
 
 
 def test_http401():
@@ -385,53 +448,89 @@ def test_http401():
     # ensure cross site scripting protection is enabled
     client.app.preferences = dict(web_ui_csrf_protection_enabled=True)
     # simulate a XSS request
-    with pytest.raises(exceptions.Unauthorized401Error):
+    with pytest.raises(exceptions.Unauthorized401Error) as exc_info:
         client.app_version(
             headers={"Origin": "https://example.com", "Referer": "https://example.com"}
         )
+    assert exc_info.value.http_status_code == 401
 
 
 @pytest.mark.parametrize("params", ({}, {"hash": "asdf"}, {"hashes": "asdf|asdf"}))
 def test_http404(client, params):
-    # 404 for a post
-    with pytest.raises(exceptions.NotFound404Error):
-        client.torrents_rename(hash="asdf", new_torrent_name="erty")
+    with pytest.raises(exceptions.NotFound404Error, match="Torrent hash") as exc_info:
+        client.torrents_rename(hash="zxcv", new_torrent_name="erty")
+    assert "zxcv" in exc_info.value.args[0]
 
-    # 404 for a get
-    with pytest.raises(exceptions.NotFound404Error):
-        client._get(
-            _name="torrents", _method="rename", params={"hash": "asdf", "name": "erty"}
-        )
-
-    response = MockResponse(status_code=404, text="")
-    with pytest.raises(exceptions.HTTPError):
+    response = MagicMock(spec=Response, status_code=404, text="")
+    with pytest.raises(exceptions.HTTPError, match="") as exc_info:
         Request._handle_error_responses(data={}, params=params, response=response)
+    assert exc_info.value.http_status_code == 404
+    if params:
+        assert params[list(params.keys())[0]] in exc_info.value.args[0]
+
+    response = MagicMock(spec=Response, status_code=404, text="unexpected msg")
+    with pytest.raises(exceptions.HTTPError, match="unexpected msg") as exc_info:
+        Request._handle_error_responses(data={}, params=params, response=response)
+    assert exc_info.value.http_status_code == 404
+    assert exc_info.value.args[0] == "unexpected msg"
+
+    response = MagicMock(spec=Response, status_code=404, text="")
+    with pytest.raises(exceptions.HTTPError, match="") as exc_info:
+        Request._handle_error_responses(data=params, params={}, response=response)
+    assert exc_info.value.http_status_code == 404
+    if params:
+        assert params[list(params.keys())[0]] in exc_info.value.args[0]
+
+    response = MagicMock(spec=Response, status_code=404, text="unexpected msg")
+    with pytest.raises(exceptions.HTTPError, match="unexpected msg") as exc_info:
+        Request._handle_error_responses(data=params, params={}, response=response)
+    assert exc_info.value.http_status_code == 404
+    assert exc_info.value.args[0] == "unexpected msg"
+
+
+def test_http405(client, app_version):
+    # v4.4.4 uses same API version as previous release...
+    if v(app_version) >= v("v4.4.4"):
+        with pytest.raises(exceptions.MethodNotAllowed405Error) as exc_info:
+            client._get(APINames.Authorization, "logout")
+        assert exc_info.value.http_status_code == 405
 
 
 def test_http409(client, app_version):
-    if v(app_version) > v("4.1.5"):
-        with pytest.raises(exceptions.Conflict409Error):
+    if v(app_version) >= v("4.1.6"):
+        with pytest.raises(exceptions.Conflict409Error) as exc_info:
             client.torrents_set_location(torrent_hashes="asdf", location="/etc/asdf/")
+        assert exc_info.value.http_status_code == 409
 
 
 def test_http415(client):
-    with pytest.raises(exceptions.UnsupportedMediaType415Error):
+    with pytest.raises(exceptions.UnsupportedMediaType415Error) as exc_info:
         client.torrents.add(torrent_files="/etc/hosts")
+    assert exc_info.value.http_status_code == 415
 
 
 @pytest.mark.parametrize("status_code", (500, 503))
 def test_http500(status_code):
-    response = MockResponse(status_code=status_code, text="asdf")
-    with pytest.raises(exceptions.InternalServerError500Error):
+    response = MagicMock(spec=Response, status_code=status_code, text="asdf")
+    with pytest.raises(exceptions.InternalServerError500Error) as exc_info:
         Request._handle_error_responses(data={}, params={}, response=response)
+    assert exc_info.value.http_status_code == status_code
+
+
+@pytest.mark.parametrize("status_code", (402, 406))
+def test_http_error(status_code):
+    response = MagicMock(spec=Response, status_code=status_code, text="asdf")
+    with pytest.raises(exceptions.HTTPError) as exc_info:
+        Request._handle_error_responses(data={}, params={}, response=response)
+    assert exc_info.value.http_status_code == status_code
 
 
 def test_request_retry_success(monkeypatch, caplog):
-    def request500(*arg, **kwargs):
+    def request500(*args, **kwargs):
         raise exceptions.HTTP500Error()
 
     client = Client(VERIFY_WEBUI_CERTIFICATE=False)
-    _ = client.app.version  # do the login first
+    client.auth_log_in()
     with monkeypatch.context() as m:
         m.setattr(client, "_request", request500)
         with caplog.at_level(logging.DEBUG, logger="qbittorrentapi"):
@@ -442,20 +541,13 @@ def test_request_retry_success(monkeypatch, caplog):
 
 def test_request_retry_skip(caplog):
     client = Client(VERIFY_WEBUI_CERTIFICATE=False)
-    _ = client.app.version  # do the login first
+    client.auth_log_in()
     with caplog.at_level(logging.DEBUG, logger="qbittorrentapi"):
         with pytest.raises(exceptions.MissingRequiredParameters400Error):
             client.torrents_rename()
     assert "Retry attempt" not in caplog.text
 
 
-@pytest.mark.parametrize("status_code", (402, 406))
-def test_http_error(status_code):
-    response = MockResponse(status_code=status_code, text="asdf")
-    with pytest.raises(exceptions.HTTPError):
-        Request._handle_error_responses(data={}, params={}, response=response)
-
-
 def test_verbose_logging(caplog):
     client = Client(VERBOSE_RESPONSE_LOGGING=True, VERIFY_WEBUI_CERTIFICATE=False)
     with caplog.at_level(logging.DEBUG, logger="qbittorrentapi"):
@@ -466,6 +558,6 @@ def test_verbose_logging(caplog):
 
 def test_stack_printing(capsys):
     client = Client(PRINT_STACK_FOR_EACH_REQUEST=True, VERIFY_WEBUI_CERTIFICATE=False)
-    _ = client.app.version
+    client.app_version()
     captured = capsys.readouterr()
     assert "print_stack()" in captured.err
